#!/usr/bin/perl
#
# Copyright (c) Ensim Corporation 2000, 2001   All Rights Reserved.
#
# This software is furnished under a license and may be used and copied
# only  in  accordance  with  the  terms  of such  license and with the
# inclusion of the above copyright notice. This software or any other
# copies thereof may not be provided or otherwise made available to any
# other person. No title to and ownership of the software is hereby
# transferred.
#
# The information in this software is subject to change without notice
# and  should  not be  construed  as  a commitment by Ensim Corporation.
# Ensim assumes no responsibility for the use or  reliability  of its
# software on equipment which is not supplied by Ensim.
#
# Exit codes (on failure error message goes to stderr):
#  0 - success
#  1 - failure
# 
# Boolean arguments on the command line are given as 0 or 1. All the
# command line arguments are encoded to avoid problems with escapes.
#
# All functions defined here either return an error message if an
# error occured and "" if everything went well or allways return a
# valid value, but exit (with code 1) printing an error message if an
# error occurs. This second type of functions have _e appended to
# their name. Functions are allowed to print results onto stdout, but
# errors are printed only in the main program.
#

package VHCommon;

require VHAppConf;
require VHConst;
require VHConfiguration;
use IO::Seekable;

#
# Processes a command line argument that is supposed to be a y/n flag
#
sub Flag_e() {
  my ($flag)=@_;
  if ($flag eq "1") {
    return 1;
  } elsif ($flag eq "0") {
    return 0;
  }
    print STDERR "Wrong argument $flag passed where a flag was expected\n";
    exit 1;
}

#
# This function deletes the filesystem used by a user with virtual
# hosting. Returns an error message that is empty on success.
#
sub DeleteFileSystem() {
  my ($user)=@_;
  my ($error_message)="";

  system("/bin/umount $VHConst::VIRTUALFS/$user/dev/pts >/dev/null 2>/dev/null");
  system ("/bin/rm -rf $VHConst::VIRTUALFS/$user >/dev/null 2>/dev/null");
  my($exit_value)  = $? >> 8;
  if ($exit_value ne 0) {
    $error_message="Error deleting virtual file system $VHConst::VIRTUALFS/$user .\n";
  }
  unlink("/etc/logrotate.d/0virtualhosting.$user") or $error_message .= "Cannot delete file /etc/logrotate.d/0virtualhosting.$user .\n";
  unlink("/etc/cron.daily/0virtualhosting.$user") or $error_message .= "Cannot delete file /etc/cron.daily/0virtualhosting.$user .\n";
  unlink("/etc/cron.weekly/0virtualhosting.$user") or $error_message .= "Cannot delete file /etc/cron.weekly/0virtualhosting.$user .\n";
  unlink("/etc/cron.monthly/0virtualhosting.$user") or $error_message .= "Cannot delete file /etc/cron.monthly/0virtualhosting.$user .\n";
  #remove all the symlinks from the user's home pointing to the
  #virtual file system
  my(@pwent) = getpwnam($user);
  unless (@pwent) {return $error_message} #Maybe the user is already gone
  my($homedir) = $pwent[7];
  if (opendir (HOMEDIR,$homedir)) { 
    my(@filenames) = map {"$homedir/$_"} readdir(HOMEDIR);
    my(@symlinks) = grep {-l $_} @filenames;
    my($link);
    foreach $link (@symlinks) { #all the symlinks
      my ($value)= readlink ($link);
      if ($value =~ /$VHConst::VIRTUALFS\/$user\//) {
	unlink($link) or $error_message .= "Cannot delete file $link .\n"
      }
    }
  } else {
    unless ($error_message) {
      $error_message.="Cannot open home directory $homedir for user $user .\n";
    }
  }
  return $error_message;
}

#
# This function adds an user to the server. The user will be
# associated a new group with the same name.
#
sub AddDomainOwner() {
  my($user,$passwd,$domain,$telnet,$ftp)=@_;
  my(@pwent) = getpwnam($user);
  if (@pwent) {
    return "The user $user already exists.\n";
  }
  my(@grent) = getgrnam($user);
  if (@grent) {
    return "A group called $user already exists.\n";
  }
  my($shell) = &DetermineShell($ftp,$telnet);
  my($error_message)="";
  unless (open (ADDUSER, "/usr/sbin/real.useradd -m -s $shell -c \"$domain\" $user 2>&1 |"))
  #Solaris: We'll see
    {return "Errors during useradd.\n"}
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {$error_message.="Errors during useradd.\n"}
  # Hack! ignore the "unknown group" error message. ???
  if ($error_message eq "useradd: unknown group \n") {
    $error_message="";
  }
  unless ($error_message) {
    unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
      {return "Cannot set password.\n"}
    print CHPASSWD "$user:$passwd\n";
    unless (close (CHPASSWD))
      {$error_message.="Errors during chpasswd.\n"}
  }

  return $error_message;
}

# add symlink for name based domain owner's mail
sub AddNBDomainOwnerSymlink() {
  my ($user)=@_;
  my ($inbox)="/var/spool/mail/$user";
  my ($vroot) = "$VHConst::VIRTUALFS/$user";
  my ($chrootinbox)="$vroot/var/spool/mail/$user";

  unless (open (INBOX,">$chrootinbox")) {
      return "Cannot create mail box $chrootinbox .\nErrors during useradd.\n";
  }
  close (INBOX);

  if (system("/bin/ln -s $chrootinbox $inbox") && 0xff00) {
    return "Cannot create mail box $inbox.\nErrors during useradd.\n"
  }
  #get uid and gid of user
  unless (open(PASSWD,"/etc/passwd")) {
    return "Cannot read password file /etc/passwd.\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  my(@goodlines) = grep {$_ =~ /^$user:/} @lines;
  unless ($#goodlines == 0) {
    return "User $user does not exist.\nErrors during useradd.\n";
  }
  my (@pwent) = split(/:/,$goodlines[0]);
  my ($uid) = ($pwent[2]);
  my ($gid) = ($pwent[3]);
  if (system("chown $uid.$gid $inbox") && 0xff00) {
    return "Cannot chown mail box $inbox .\nErrors during useradd.\n";
  }
  unless (chmod(0600,$inbox)) {
    return "Cannot chmod mail box $inbox .\nErrors during useradd.\n";
  }
  return "";
}

# deletes a domain owner user (and its group) from the server
sub DelDomainOwner() {
  my ($user)=@_;

  my($error_message)="";
  unless (open (DELUSER, "/usr/sbin/userdel -r $user 2>&1 |"))
    {return "Errors during userdel.\n"}
  while (<DELUSER>) {
    $error_message.=$_;
  }
  close (DELUSER) or $error_message.="Errors during userdel.\n";
  my ($inbox)="/var/spool/mail/$user";
  if (stat($inbox)) {
    unlink ($inbox) or $error_message.= "Cannot delete inbox $inbox .\n";
  }
  return $error_message;
}

# deletes symlink for name based domain owner's mail
sub DelNBDomainOwnerSymlink() {
  my ($user)=@_;
  my ($error_message)="";
  my ($inbox)="/var/spool/mail/$user";
  unlink ($inbox) or $error_message= "Cannot delete inbox $inbox .\n";
  if ($error_message) {print STDERR $error_message;}
  return "";
}

# add an apache group to a chroot environment
sub AddApacheGroup() {
  my ($group,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";

  my(@pwent) = getpwnam($record->{User});  #Get the user info of the
                                           #virtual domain owner
  unless(@pwent) {return "Owner of domain $Domain , $record->{User} does not exist.\n";}
  my ($error_message) = &ActualAddApacheGroup($group,$vroot);
  return $error_message;
}

# add a user to a chroot environment
sub AddVirtUser() {
  my ($user,$fullname,$ftpallowed,$telnetallowed,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";

  my(@pwent) = getpwnam($record->{User});  #Get the user info of the
                                           #virtual domain owner
  unless(@pwent) {return "Owner of domain $Domain , $record->{User} does not exist.\n";}
  my ($gid) = ($pwent[3]);
  my ($shell) = &VHCommon::DetermineShell($ftpallowed,$telnetallowed);
  my ($error_message) = &ActualAddVirtUser($user,$fullname,$shell,$gid,$vroot);
  if ($error_message =~ /^useradd: can\'t get unique uid/) {
    #it might make sense to switch to another virtual uid range
    my ($err);
    if ($err = &VHConfiguration::ReadUIDConfFile()) {
      return $error_message.$err;
    }
    if ($#{$record->{Ranges}}>=0) {
      #OK, we have uid ranges to choose from
      if ($err = &VHConfiguration::ParseDomainPasswd ($record)) {
	return $error_message.$err;
      }
      if ($err = &VHConfiguration::SetLeastUsedRange ($record)) {
	return $error_message.$err;
      }
      return &ActualAddVirtUser($user,$fullname,$shell,$gid,$vroot);
    }
  }
  return $error_message;
}

# for name-based domains; add user to server
sub AddNBVirtUser() {
  my ($user,$fullname,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  #get uid and gid of user in chrooted environment
  unless (open(PASSWD,"$vroot/etc/passwd")) {
      return "Cannot read password file $vroot/etc/passwd.\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  my(@goodlines) = grep {$_ =~ /^$user:/} @lines;
  unless ($#goodlines == 0) {
      return "User $user of domain $record->{Domain} does not exist.\nErrors during useradd.\n";
  }
  my (@pwent) = split(/:/,$goodlines[0]);
  my ($uid) = ($pwent[2]);
  my ($gid) = ($pwent[3]);
  my ($shell) = &DetermineShell(0,0);
  #We add the user as user.siteownername
  if($error_message = &ActualAddNBVirtUser("$user\.$record->{User}",$fullname,$shell,$uid,$gid, $vroot)) {
      return $error_message;
  }
  SyncNBUser("$user\.$record->{User}", "$user", $record);
  my ($inbox)="/var/spool/mail/$user\.$record->{User}";
  my ($chrootinbox)="$vroot/var/spool/mail/$user";
  if (system("/bin/ln -s $chrootinbox $inbox") && 0xff00) {
    return "Cannot create mail box $inbox.\nErrors during useradd.\n" 
  }
  #somehow I have to make system call to chown symlink
  if (system("chown $uid.$gid $chrootinbox") && 0xff00) {
    return "Cannot chown mail box $chrootinbox .\nErrors during useradd.\n";
  }
  unless (chmod(0600,$chrootinbox)) {
    return "Cannot chmod mail box $chrootinbox .\nErrors during useradd.\n";
  }
  return $error_message;
}

sub SyncNBUser() {
    # syncs chrooted pwd to top level pwd.
    # assumes user already exists in chrooted env.
    my ($user,$chrootuser,$record)=@_;
    my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
    unless (open(SHADOW,"$vroot/etc/shadow")) {
        return "Cannot open shadow file $vroot/etc/shadow.\n";
    }
    my (@shadow_lines)=<SHADOW>;
    close (SHADOW);
 
    my(@goodshadow_lines) = grep {$_ =~ /^$chrootuser:/} @shadow_lines;
    unless ($#goodshadow_lines == 0) {
         return "User $chrootuser of domain $record->{Domain} does not exist.\n";
    }
 
    my (@shadow_pwent) = split(/:/,$goodshadow_lines[0]);
    my ($shadow_pwd) = ($shadow_pwent[1]);
    unless (open(CHPASSWD,"|/usr/sbin/chpasswd -e")) {
         return "Could not change /etc/shadow password for user $user.\n";
    }
    print CHPASSWD "$user:$shadow_pwd\n";
    close CHPASSWD;
}

sub ActualAddApacheUser() {
  my($user,$group,$fullname,$shell,$gid,$vroot)=@_;
  # add apache user
  my($error_message)="";
  unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -m -s $shell -g $gid -c \"$fullname\" $user 2>&1 |"))
  #Solaris: unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -s $shell -u uid -g $gid -c \"$fullname\" $user 2>&1 |"))
    {return "Cannot create the chrooted useradd process.\nErrors during useradd.\n"}
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {$error_message.="Errors during useradd.\n"}
  if ($error_message) {
      return $error_message;
  }
  unless (chmod(0711,"$vroot/home/$user")) {
      return "Cannot change permissions on the user's home directory.\n";
  }
  my ($inbox)="$vroot/var/spool/mail/$user";
  unless (open (INBOX,">$inbox")) {
      return "Cannot create mail box $inbox .\nErrors during useradd.\n";
  }
  close (INBOX);
  unless (open(PASSWD,"$vroot/etc/passwd")) {
      return "Cannot read password file $vroot/etc/passwd .\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  @lines = grep {$_ =~ /^$user:/} @lines;
  unless ($#lines == 0) {
      return "User not properly added to password file.\nErrors during useradd.\n";
  }

  my (@pwent)=split(/:/,$lines[0]);
  unless (chown($pwent[2],$pwent[3], $inbox)) {
      return "Cannot chown mail box $inbox .\nErrors during useradd.\n";
  }
  unless (chmod(0600,$inbox)) {
      return "Cannot chmod mail box $inbox .\nErrors during useradd.\n";
  }
  return "";
}

sub ActualAddVirtUser() {
  my($user,$fullname,$shell,$gid,$vroot)=@_;
  # add user
  my($error_message)="";
  unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -m -s $shell -g $gid -c \"$fullname\" $user 2>&1 |"))
  #Solaris: unless (open (ADDUSER, "/usr/sbin/chroot $vroot /usr/sbin/useradd -s $shell -u uid -g $gid -c \"$fullname\" $user 2>&1 |"))
    {return "Cannot create the chrooted useradd process.\nErrors during useradd.\n"}
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {$error_message.="Errors during useradd.\n"}
  if ($error_message) {
      return $error_message;
  }
  unless (chmod(0711,"$vroot/home/$user")) {
      return "Cannot change permissions on the user's home directory.\n";
  }
  my ($inbox)="$vroot/var/spool/mail/$user";
  unless (open (INBOX,">$inbox")) {
      return "Cannot create mail box $inbox .\nErrors during useradd.\n";
  }
  close (INBOX);
  unless (open(PASSWD,"$vroot/etc/passwd")) {
      return "Cannot read password file $vroot/etc/passwd .\nErrors during useradd.\n";
  }
  my (@lines)=<PASSWD>;
  close (PASSWD);
  @lines = grep {$_ =~ /^$user:/} @lines;
  unless ($#lines == 0) {
      return "User not properly added to password file.\nErrors during useradd.\n";
  }

  my (@pwent)=split(/:/,$lines[0]);
  unless (chown($pwent[2],$pwent[3], $inbox)) {
      return "Cannot chown mail box $inbox .\nErrors during useradd.\n";
  }
  unless (chmod(0600,$inbox)) {
      return "Cannot chmod mail box $inbox .\nErrors during useradd.\n";
  }
  return "";
}

# for name-based domains; add user as username.domain with same uid
sub ActualAddNBVirtUser() {
  my($user,$fullname,$shell,$uid,$gid, $vroot)=@_;
  my($error_message)="";
  unless (open (ADDUSER, "/usr/sbin/real.useradd -M -s $shell -g $gid -u $uid -c \"$fullname\" $user 2>&1 |"))
    {return "Cannot create the useradd process on server.\nErrors during useradd.\n"}
  while (<ADDUSER>) {
    $error_message.=$_;
  }
  unless (close (ADDUSER))
    {$error_message.="Errors during useradd on server.\n"}
  if ($error_message) {
      return $error_message;
  }

  return "";
}

# delete a user from a chroot environment
sub DelVirtUser() {
  my ($user,$record)=@_;
  my ($error_message)="";
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  #Here we do an otherwise useless check to avoid letting the domain
  #owners to delete important "virtual users" brought over from the
  #server (root, ftp, majordomo, domain owner, etc)
  unless(open (PASSWD, "$vroot/etc/passwd")) {
    return "Cannot read virtual password file $vroot/etc/passwd .\n";
  }
  my($passwdline);
  while (<PASSWD>) {
      if ($_ =~ /^$user:/ ) {
	  $passwdline = $_;
	  last;
      }
  }
  close (PASSWD);
  if (not $passwdline) {
      #We should never get to this line
      return "User $user not found in site $record->{Domain} .\n";
  }
  my(@pwent) = split(/:/,$passwdline);
  my($uid) = $pwent[2];
  if($error_message=&VHConfiguration::ReadUIDConfFile()) {
      return $error_message
  }
  if($uid<$VHConfiguration::MinVirtUid or $uid>$VHConfiguration::MaxVirtUid) {
      return "You are not allowed to delete special user $user .\n";
  }
  #End of stupid check
  unless (open (DELUSER, "/usr/sbin/chroot $vroot /usr/sbin/userdel -r $user 2>&1 |"))
    {return "Cannot create the chrooted deluser process.\n"}
  while (<DELUSER>) {
    $error_message.=$_;
  }
  close (DELUSER) or $error_message.="Errors during userdel.\n";
  my ($inbox)="$vroot/var/spool/mail/$user";
  if (stat($inbox)) {
    unlink ($inbox) or $error_message.= "Cannot delete inbox $inbox .\n";
  }
  print STDERR $error_message;
  #Now we delete the aliases that point to this user
  if (open (ALIAS,"$vroot/etc/aliases")) {
      my (@lines);
      @lines = <ALIAS>;
      close (ALIAS);
      my ($linenum);
      $linenum = $#lines;
      @lines = grep {$_ !~ /\s$user$/} @lines;
      if ($linenum != $#lines) {
          if (open (ALIAS,">$vroot/etc/aliases")) {
              print ALIAS @lines;
              close (ALIAS);
          } else {
              print STDERR "Cannot write to alias file $vroot/etc/aliases .\n";
          }
      }
  } else {
      print STDERR "Cannot read alias file $vroot/etc/aliases .\n";
  }

  return "";
}


# for name-based domains; delete a user from the server
sub DelNBVirtUser() {
  my ($user,$record)=@_;
  my ($error_message)="";
  #We have user.siteownername
  if ($error_message = &VHCommon::SwitchToDotUser(\$user,$record)) {return $error_message}
  unless (open (DELUSER, "/usr/sbin/userdel $user 2>&1 |"))
    {return "Cannot create the deluser process.\n"}
  while (<DELUSER>) {
    $error_message.=$_;
  }
  close (DELUSER) or $error_message.="Errors during userdel.\n";

  #Remove the symlink to the virtual user's mail box 
  my ($inbox)="/var/spool/mail/$user";
  if (stat($inbox)) {
    unlink ($inbox) or $error_message.= "Cannot delete inbox $inbox .\n";
  }
  if ($error_message) {print STDERR $error_message;}
  return "";
}

sub SwitchToDotUser() {
    my($puser,$record)=@_;
    $$puser = "$$puser\.$record->{User}";
    return "";
}

# change the password of a virtual user
sub ChpasswdVirtUser() {
  my ($user,$passwd,$record)=@_;
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  my ($error_message)="";
  unless(open (CHPASSWD, "|/usr/sbin/chroot $vroot /usr/sbin/chpasswd"))
    {return "Cannot create the chrooted chpasswd process.\n"}
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    {$error_message.="Errors during chpasswd in chrooted.\n"}
  # for name-based domains; change the password of a user on server
  if ($record->{DomainType} eq "NameBased" and $record->{IMAP}) {
      unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
        {return "Cannot creat the chpasswd process.\n"}
      print CHPASSWD "$user\.$record->{User}:$passwd\n";
      unless(close (CHPASSWD))
        {$error_message.="Errors during chpasswd on server.\n"}
  }
  return $error_message;
}

# change the password of a user from a chroot environment
sub ChpasswdDomain() {
  my ($record,$passwd)=@_;
  my ($user) = $record->{User};

  #Change it on the server first
  unless(open (CHPASSWD, "|/usr/sbin/chpasswd"))
    {return "Cannot create the chpasswd process.\n"}
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    {return "Errors during chpasswd.\n"}

  #and in the chrooted environment afterwards
  my ($vroot) = "$VHConst::VIRTUALFS/$record->{User}";
  unless(open (CHPASSWD, "|/usr/sbin/chroot $vroot /usr/sbin/chpasswd")){
      print STDERR "Cannot create the chrooted chpasswd process.\n";
      return "";
  }
  print CHPASSWD "$user:$passwd\n";
  unless (close (CHPASSWD))
    {print STDERR "Errors during chpasswd for the virtual user.\n"}

  #and if frontpage is installed, change it there too
  if ($record->{FPExt}) {
      if ($error_message = &VHAppConf::resetFrontPagePassword($record->{User})) {
          return "Failed to change the associated frontpage password\n".$error_message;
      }
  }

  return "";
}

# Gives he shell a virtual user gets based on its flags
sub DetermineShell() {
#  my ($email,$ftp)=@_;
#  return "/bin/".($email?($ftp?"emailftp":"email"):($ftp?"ftp":"false"));
  my ($ftp,$telnet)=@_;
  return ($telnet?($ftp?"/bin/bash":"/bin/telnetonly"):($ftp?"/bin/ftponly":"/bin/nologinshell"));
}

# Formats a line from the virtual passwd file
sub FormatVirtUser() {
  my ($telnetenabled,$ftpenabled,$domaintype,$line,$used,$quota)=@_;
  chop ($line);
  my (@fields) = split(/:/,$line);
  my ($shell) = $fields[6];
  my ($ftp) = ((grep {$shell eq $_}("/bin/telnetonly","/bin/nologinshell","/bin/false","/dev/null"))?0:1);
  if ($domaintype eq "NameBased") {$ftp=0;}
  if (!$ftpenabled) {$ftp=0;}
  my ($telnet) = (($shell eq "/bin/bash")?"1":"0");
  if (!$telnetenabled) {$telnet=0;}
  if (not defined $quota) {$quota="";}
  if (not defined $used) {$used="";}
  my($uid) = $fields[2];
  my($error_message,$privileged);
  if($error_message=&VHConfiguration::ReadUIDConfFile()) {
      return $error_message;
  }
  if($uid<$VHConfiguration::MinVirtUid or $uid>$VHConfiguration::MaxVirtUid) {
      $privileged=1;
  } else {
      $privileged=0;
  }
  return "$fields[0]:$ftp:$telnet:$privileged:$quota:$used:$fields[4]\n";
}

#
# This little shrewd function cleans up a virtual domain so that
# telnet can be disabled. For this, it kills all the processes
# belonging to virtual users who telneted in. After this it unmounts
# /dev/pts so that noone else gets in.
#
sub PrepareDisableTelnet() {
    my ($dir)=@_;
    my (@victims)=();
    open(PROCESSES,'/bin/ps -Ao "%p %y"|') or 
      return "Cannot do ps.\n";
    while(<PROCESSES>){
	my(@list)=split(/\s+/,$_);
	my($pid,$tty)=grep {$_} @list;
	if ($tty =~ /^$dir/) {
	    push @victims,$pid;
	}
    }
    kill 9,@victims;
    system ("/bin/umount $dir/dev/pts >/dev/null 2>/dev/null");
    if (($? >> 8) != 0) {
	return "Error unmounting file system $dir/dev/pts .\n";
    }
    return"";
}

#
# This function checks if all the services marked as on for a virtual
# domain are actually supported.
#
sub CheckSupportedServices() {
    my ($record)=@_;

    my($analog_supported,$ssl_supported,$imap_supported,$vacation_supported,
       $majordomo_supported,$sendmail_supported,$telnet_supported,
       $bind_supported);
    open (SVCINST,"/etc/virtualhosting/supported_services") or
      return "Cannot open service installation witness file.\n";
    open (SVCCRT,"/usr/bin/wpsvcquery -a|") or
      return "Cannot run /usr/bin/wpsvcquery .\n";
    my (@svcinst)=<SVCINST>;
    my (@svccrt)=<SVCCRT>;
    close (SVCINST);
    close (SVCCRT);
    if ((grep /^analog\s/,@svcinst) and (grep /^analog\n$/,@svccrt)) {
	$analog_supported = 1;
    } else {
	$analog_supported = 0;
    }
    if ((grep /^openssl\s/,@svcinst) and (grep /^openssl\n$/,@svccrt)) {
	$ssl_supported = 1;
    } else {
	$ssl_supported = 0;
    }
#    if ((grep /^sslc\s/,@svcinst) and (grep /^sslc\n$/,@svccrt)) {
#	$ssl_supported = 1;
#    } else {
#	$ssl_supported = 0;
#    }
    if ((grep /^imap\s/,@svcinst) and (grep /^imap\n$/,@svccrt)) {
	$imap_supported = 1;
    } else {
	$imap_supported = 0;
    }
    if ((grep /^majordomo\s/,@svcinst) and (grep /^majordomo\n$/,@svccrt)) {
	$majordomo_supported = 1;
    } else {
	$majordomo_supported = 0;
    }
    if ((grep /^vacation\s/,@svcinst) and (grep /^vacation\n$/,@svccrt)) {
	$vacation_supported = 1;
    } else {
	$vacation_supported = 0;
    }
    if ((grep /^sendmail\s/,@svcinst) and (grep /^sendmail\n$/,@svccrt)) {
	$sendmail_supported = 1;
    } else {
	$sendmail_supported = 0;
    }
    if ((grep /^bind\s/,@svcinst) and (grep /^bind\n$/,@svccrt)) {
	$bind_supported = 1;
    } else {
	$bind_supported = 0;
    }
    $telnet_supported = 1;
    my ($error_message)="";
    if ($record->{Analog} and not $analog_supported) {
	$error_message .= "Option Analog not supported\n";
    }
    if ($record->{SSL} and not $ssl_supported) {
	$error_message .= $record->{SSL} . "$ssl_supported". "Option SSL not supported\n";
    }
    if ($record->{Majordomo} and not $majordomo_supported) {
	$error_message .= "Option Majordomo not supported\n";
    }
    if ($record->{Vacation} and not $vacation_supported) {
	$error_message .= "Option Vacation not supported\n";
    }
    if ($record->{Sendmail} and not $sendmail_supported) {
	$error_message .= "Option Sendmail not supported\n";
    }
    if ($record->{Telnet} and not $telnet_supported) {
	$error_message .= "Option Analog not supported\n";
    }
    if ($bind_supported) {
	$record->{Nslookup} = 1;
    } else {
	$record->{Nslookup} = 0;
    }
    return $error_message;
}

#
# This function checks if a service is installed and supported on the server
#
sub IsServiceInstalledSupported() {
    my ($service)=@_;
    # when we standardize the naming on backend/frontend,we'll be able
    # to get rid of this junk...
    my ($svc_name); # name of service as it is running on server
    if ($service eq "Web") { $svc_name="apache"; }
    elsif ($service eq "Analog") { $svc_name="analog"; }
    elsif ($service eq "Nslookup") { $svc_name="bind"; }
    elsif ($service eq "FPext") { $svc_name="frontpage"; }
    elsif ($service eq "Sendmail") { $svc_name="sendmail"; }
    elsif ($service eq "IMAP") { $svc_name="imap"; }
    elsif ($service eq "Majordomo") { $svc_name="majordomo"; }
    elsif ($service eq "FTP") { $svc_name="proftpd"; }
    elsif ($service eq "SSL") { $svc_name="openssl"; }
    elsif ($service eq "Vacation") { $svc_name="vacation"; }
    elsif ($service eq "Telnet") { $svc_name="telnet"; }
    elsif ($service eq "MivaMerchant") { $svc_name="mivamerchant_unl"; }
    else { $svc_name=$service; }
    open (SVCINST,"/etc/virtualhosting/supported_services") or
      return "Cannot open service installation witness file.\n";
    open (SVCCRT,"/usr/bin/wpsvcquery -a|") or
      return "Cannot run /usr/bin/wpsvcquery .\n";
    my (@svcinst)=<SVCINST>;
    my (@svccrt)=<SVCCRT>;
    close (SVCINST);
    close (SVCCRT);
    if ((grep /^$svc_name\s/,@svcinst) and (grep /^$svc_name\n$/,@svccrt)) {
	return "";
    } else {
	return "Option $service not supported.\n";
    }
}

return 1;

